
#ifndef  __SHADER_VIEWPORT__H_
#define  __SHADER_VIEWPORT__H_

#include <FL/Fl_Gl_Window.H>

// this includes gl.h
#include "records.h"

#define	NPOINTS		32
#define NPOINTS1	33

class GuiBuilder;
class Fl_Select_Browser;

class ShaderViewport : public Fl_Gl_Window
{
public:
	ShaderViewport(int x, int y, int w, int h, const char *label = 0);
	virtual ~ShaderViewport();

	virtual void draw();
	virtual int handle(int event);

	void onMouseDown();
	void onMouseDrag();
	void onMouseRelease();
	void onMouseWheel();

	void onAddVertexShader();
	void onRemoveVertexShader();

	void onAddFragmentShader();
	void onRemoveFragmentShader();
	void onCompile();
	void onUseProgram();

	void onEditUniform();
	void onLoadUniforms();
	void onSelectUniform();
	void onUniformIndex();

	void onGlobalAmbient();

	void onLocalViewer();
	void onTwoSided();
	void onColorControl();
	void onGlobalLighting();

	void onAddLight();
	void onRemoveLight();
	void onLightSelection();

	void onLightPosition();
	void onLightAmb();
	void onLightDiffuse();
	void onLightSpec();
	void onLightSpotDir();
	void onLightSpotExponent();
	void onLightSpotCutoff();
	void onLightAttenConst();
	void onLightAttenLinear();
	void onLightAttenQuad();
	void onLightEnable();
	void onLightShow();

	void onMatAmb();
	void onMatDiffuse();
	void onMatSpec();
	void onMatEmit();
	void onMatShin();

	void onMatAmbBack();
	void onMatDiffuseBack();
	void onMatSpecBack();
	void onMatEmitBack();
	void onMatShinBack();
	void onSameAsFront();

	void onAddTexture();
	void onRemoveTexture();
	void onTextureSelection();
	void onMagFilter();
	void onMinFilter();
	void onTextureMode();
	void onInternalFormat();
	void onWrapS();
	void onWrapT();
	void onBorderColor();
	void onBlendColor();
	void onTexEnabled();


	void onChangeShape();

	void setGuiBuilder(GuiBuilder *b);

private:
	void glInit();

	void onAddShader(GLenum type, Fl_Select_Browser *browser);
	void onRemoveShader(GLenum type, Fl_Select_Browser *browser);

	void updateUniforms();
	void clearUniforms();

	template <class T>
	int findAvailableIndex(GLint max, Fl_Select_Browser *browser);

	void updateLightData(int browserIndex);
	void updateTextureData(int browserIndex);
	void updateMaterialData();

	void drawLight(GLfloat pos[4]);
	void drawCircle();
	void drawSphere();
	void drawPlane();
	void textureCoords(float s, float t);

	GLenum lightGLIndex(int pos);
	GLenum textureGLIndex(int pos);
	GLenum materialFace();

	GLint compileShader(ShaderRecord *r);

	GuiBuilder *builder;

	Type2NameMap	typeMap;

	float	longitude;
	float	latitude;
	float	radius;

	float	cosines[NPOINTS1];
	float	sines[NPOINTS1];

	int lastX, lastY;

	GLuint program;
	GLint  maxLights, maxTextures;
	bool firstInit;
	bool progValid;
};

#endif  //__SHADER_VIEWPORT__H_

