
#include "magickImage.h"

bool MagickImage::initialized = false;
unsigned MagickImage::refs = 0;

MagickImage::MagickImage()
	: wand(0)
	, pixels(0)
{
	if(!initialized)
	{
		MagickWandGenesis();
	}

	wand = NewMagickWand();

	++refs;
}

MagickImage::MagickImage(MagickImage const & other)
	: wand(CloneMagickWand(other.wand))
	, pixels(0)
{
}

MagickImage::~MagickImage()
{
	--refs;

	if(wand)
	{
		DestroyMagickWand(wand);
		wand = 0;
	}

	if(refs == 0)	MagickWandTerminus();

	if(pixels)	delete [] pixels;
}

bool MagickImage::loadImage(char const *fname)
{
	MagickBooleanType res = MagickReadImage(wand, fname);
	bool ret = false;

	if(res == MagickTrue)
	{
		ret = true;
	}

	return ret;
}

unsigned long MagickImage::height()
{
	return MagickGetImageHeight(wand);
}

unsigned long MagickImage::width()
{
	return MagickGetImageWidth(wand);
}

unsigned char *MagickImage::getPixels()
{
	if(pixels == 0)
	{
		pixels = new unsigned char[4*width()*height()];

		MagickBooleanType res;
		res = MagickGetImagePixels( wand, 0, 0, width(), height()
								  , "RGBA", CharPixel, pixels);
	}

	return pixels;
}

void MagickImage::resize(int w, int h)
{
	if(pixels)	{	delete [] pixels;	pixels = 0;	}

	MagickResizeImage(wand, w, h, CubicFilter, 1.0);
}

MagickImage *MagickImage::copy()
{
	MagickImage *newIm = new MagickImage(*this);

	return newIm;
}
